// Copyright 2009,2010 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <QtScript>
#include "capiservergame.h"
#include "capiserver.h"

class CapiServer;

CapiServerGame::CapiServerGame(int id) : CapiGame(id) {
	maxHouses = 999;
	maxHotels = 999;
	startmoney = 1500;
	collectFine = true;
	shuffleCards = false;
	auctions = true;
	doubleMoneyOnExactLanding = false;
	unlimitedHouses = false;
	jailNoRent = true;
	autoTax = false;
	spectatorsAllowed = false;
	bgColor = "";
	goId = 0;

	turnState = -1;
	auctionState = 0;
	auctionid = -1;
	auctionStatus = 0;
	auctionActor = 0;
	auctionEstate = 0;
	auctionHighBidder = 0;
	auctionHighestBid = 0;
	statusCount = 0;

	playerOnTurn = 0;

	deptRound = 0;
	replaceMath = "";

	tradeid = 1;
}

CapiServerGame::~CapiServerGame() {
	while (trades.size() > 0) {
		CapiTrade* t = trades.value(0);
		trades.removeAll(t);
		delete t;
	}
}

QString CapiServerGame::maskXML(QString txt) {
	QString msg = txt.replace("&", "&#38;");
	msg = msg.replace("<", "&#60;");
	msg = msg.replace(">", "&#62;");
	msg = msg.replace("=", "&#61;");
	msg = msg.replace("\"", "&#34;");
	return msg;
}

int CapiServerGame::getMaxHouses() {
	if (unlimitedHouses) return 99999;
	return maxHouses;
}

int CapiServerGame::getMaxHotels() {
	if (unlimitedHouses) return 99999;
	return maxHouses;
}

int CapiServerGame::getStartMoney() {
	return startmoney;
}

bool CapiServerGame::getCollectFine() {
	return collectFine;
}

bool CapiServerGame::getShuffleCards() {
	return shuffleCards;
}

bool CapiServerGame::getAuctions() {
	return auctions;
}

bool CapiServerGame::getDoubleMoneyOnExactLanding() {
	return doubleMoneyOnExactLanding;
}

bool CapiServerGame::getUnlimitedHouses() {
	return unlimitedHouses;
}

bool CapiServerGame::getJailNoRent() {
	return jailNoRent;
}

bool CapiServerGame::getAutoTax() {
	return autoTax;
}

bool CapiServerGame::getSpectatorsAllowed() {
	return spectatorsAllowed;
}

bool CapiServerGame::getCanBeJoined() {
	return ((players.count() < maxPlayer) && (status == "config"));
}

void CapiServerGame::setMaxHouses(int maxH) {
	maxHouses = maxH;
}

void CapiServerGame::setMaxHotels(int maxH) {
	maxHotels = maxH;
}

void CapiServerGame::setStartMoney(int startM) {
	startmoney = startM;
}

void CapiServerGame::setCollectFine(bool cf) {
	collectFine = cf;
}

void CapiServerGame::setShuffleCards(int sc) {
	shuffleCards = sc;
}

void CapiServerGame::setAuctions(int a) {
	auctions = a;
}

void CapiServerGame::setDoubleMoneyOnExactLanding(int dm) {
	doubleMoneyOnExactLanding = dm;
}

void CapiServerGame::setUnlimitedHouses(int uh) {
	unlimitedHouses = uh;
}

void CapiServerGame::setJailNoRent(int jnr) {
	jailNoRent = jnr;
}

void CapiServerGame::setAutoTax(int at) {
	autoTax = at;
}

void CapiServerGame::setSpectatorsAllowed(int se) {
	spectatorsAllowed = se;
}

void CapiServerGame::tick() {
	//this method is called every second

	if (auctionid != -1) {
		if (auctionHighestBid > 0) statusCount++;
		if (statusCount > 5) {
			auctionStatus++;
			if (auctionStatus <= 3) {
				statusCount = 0;
				sendToAll(auctionUpdate("status"));
				if (auctionStatus == 3) {
					//Auction ended
					sendToAll(createDisplayMsg(QString("%1 won the auction for %2").arg(auctionHighBidder->getName(), auctionEstate->getName()), playerOnTurn->getPos()));
					waitForAuctionPay();
				}
			} else {
				auctionStatus = 3;
			}
		}
	}
}

//Returns the Estate with given id. If no such field exists, one wil be createed and returned
CapiServerEstate* CapiServerGame::getServerEstate(int id, bool create) {
	for (int i = 0; i < estates.size(); i++)
		if (estates.value(i)->getId() == id) return (CapiServerEstate*) estates.value(i);

	if (create) {
		CapiServerEstate* f = new CapiServerEstate(id);
		estates.append(f);
		return f;
	} else {
		return 0;
	}
}

//Returns the Estate with given id. If no such field exists, one wil be createed and returned
CapiServerGroup* CapiServerGame::getServerGroup(int id, QString name, bool create) {
	foreach (CapiGroup* g, groups)
		if (g->getId() == id) return (CapiServerGroup*) g;

	if (!create) return 0;

	CapiServerGroup* g = new CapiServerGroup(id, name);
	groups.append(g);
	return g;
}

CapiServerCard* CapiServerGame::getServerCard(int id, bool create) {
	foreach (CapiCard* c, cards) {
		if (c->getId() == id) return (CapiServerCard*) c;
	}
	if (!create) return 0;

	CapiServerCard* c = new CapiServerCard(id);
	cards.append(c);
	return c;
}

QString CapiServerGame::getUpdate(QString type) {
	QString ret = "<gameupdate ";
	int m = -1;
	if (getMaster() != 0) m = getMaster()->getId();

	ret += "gameid=\""+QString::number(getId())+"\" ";

	if ((type == "") || (type.contains("gametype")))        ret += QString("gametype=\"%1\" ").arg(getType());
	if ((type == "") || (type.contains("name")))            ret += QString("name=\"%1\" ").arg(maskXML(getName()));
	if ((type == "") || (type.contains("description")))     ret += QString("description=\"%1\" ").arg(maskXML(getDesc()));
	if ((type == "") || (type.contains("status")))          ret += QString("status=\"%1\" ").arg(getStatus());
	if ((type == "") || (type.contains("players")))         ret += QString("players=\"%1\" ").arg(players.count());
	if ((type == "") || (type.contains("canbejoined")))     ret += QString("canbejoined=\"%1\" ").arg(getCanBeJoined());
	if ((type == "") || (type.contains("minplayers")))      ret += QString("minplayers=\"%1\" ").arg(getMinPlayer());
	if ((type == "") || (type.contains("mayplayers")))      ret += QString("maxplayers=\"%1\" ").arg(getMaxPlayer());
	if ((type == "") || (type.contains("master")))          ret += QString("master=\"%1\" ").arg(m);
	if ((type == "") || (type.contains("allowestatesales"))) ret += QString("allowestatesales=\"%1\" ").arg(getSellEnabled());
	ret += "/>";
	return ret;
}

void CapiServerGame::parseGame(QDomDocument doc) {
	QDomElement root = doc.documentElement();
	QDomElement e = root.firstChildElement();
	while (!e.isNull()) {
		if (e.nodeName() == "general") parseGeneralData(e);
		else if (e.nodeName() == "board") parseBoardData(e);
		else if (e.nodeName() == "cardgroup") parseCardgroup(e);
		else if (e.nodeName() == "estategroup") parseEstategroup(e);
		else if (e.nodeName() == "estate") parseEstate(e);
		else qDebug() << "Unknon Object: " << e.nodeName() << "\n";
		e = e.nextSiblingElement();
	}
}

void CapiServerGame::parseGeneralData(QDomElement e) {
	QDomElement f = e.firstChildElement();
	while (!f.isNull()) {
		if (f.nodeName() == "name") setName(f.text());
		else if (f.nodeName() == "description") setDesc(f.text());
		else if (f.nodeName() == "minplayers") setMinPlayer(f.text().toInt());
		else if (f.nodeName() == "maxplayers") setMaxPlayer(f.text().toInt());
		else if (f.nodeName() == "houses") setMaxHouses(f.text().toInt());
		else if (f.nodeName() == "hotels") setMaxHotels(f.text().toInt());
		else if (f.nodeName() == "startmoney") setStartMoney(f.text().toInt());
		else qDebug() << "Unknon General data: " << f.nodeName() << "\n";

		f = f.nextSiblingElement();
	}
}

void CapiServerGame::parseBoardData(QDomElement e) {
	QDomElement f = e.firstChildElement();
	while (!f.isNull()) {
		if (f.nodeName() == "bgcolor") bgColor = f.text();
		else if (f.nodeName() == "go") goId = f.text().toInt();
		else qDebug() << "Unknon Board data: " << f.nodeName() << "\n";

		f = f.nextSiblingElement();
	}
}

void CapiServerGame::parseCardgroup(QDomElement e) {
	QString grName = e.attribute("name", "");
	QDomElement c = e.firstChildElement();
	while (!c.isNull()) {
		CapiServerCard* sc = getServerCard(cards.size()+1);
		sc->setOwner(0);
		sc->setGroup(grName);

		QDomElement cp = c.firstChildElement();
		while (!cp.isNull()) {
			if (cp.nodeName() == "text") sc->setText(cp.text());
			else if (cp.nodeName() == "advancetonextof") sc->setAdvanceToNextOf(cp.text());
			else if (cp.nodeName() == "rentmath") sc->setRentMath(cp.text());
			else if (cp.nodeName() == "tojail") sc->setToJail(cp.text() == "1");
			else if (cp.nodeName() == "advanceto") sc->setAdvanceTo(cp.text().toInt());
			else if (cp.nodeName() == "advance") sc->setAdvance(cp.text().toInt());
			else if (cp.nodeName() == "pay") sc->setPay(cp.text().toInt());
			else if (cp.nodeName() == "payeach") sc->setPayEach(cp.text().toInt());
			else if (cp.nodeName() == "payhouse") sc->setPayHouse(cp.text().toInt());
			else if (cp.nodeName() == "payhotel") sc->setPayHotel(cp.text().toInt());
			else if (cp.nodeName() == "canbeowned") sc->setCanBeOwned(cp.text() == "1");
			else if (cp.nodeName() == "outofjail") sc->setOutOfJail(cp.text() == "1");
			else if (cp.nodeName() == "roll") sc->setRoll(cp.text() == "1");
			else if (cp.nodeName() == "title") sc->setTitle(cp.text());
			else qDebug() << "Unknon Card property: " << cp.nodeName() << "\n";

			cp = cp.nextSiblingElement();
		}

		c = c.nextSiblingElement();
	}
}

void CapiServerGame::parseEstategroup(QDomElement e) {
	QString grName = e.attribute("name", "");
	int id = e.attribute("id", "0").toInt();
	CapiServerGroup* g = getServerGroup(id, grName);

	QDomElement f = e.firstChildElement();
	while (!f.isNull()) {
		if (f.nodeName() == "color") g->setColor(f.text());
		else if (f.nodeName() == "bgcolor") g->setBgColor(f.text());
		else if (f.nodeName() == "houseprice") g->setHousePrice(f.text().toInt());
		else if (f.nodeName() == "price") g->setPrice(f.text().toInt());
		else if (f.nodeName() == "rentmath") g->setRentMath(f.text());
		else qDebug() << "Unknon Estategroup property: " << f.nodeName() << "\n";

		f = f.nextSiblingElement();
	}
}

void CapiServerGame::parseEstate(QDomElement e) {
	QString esName = e.attribute("name", "");
	int id = e.attribute("id", "0").toInt();
	CapiServerEstate* es = getServerEstate(id, true);
	es->setOwner(0);
	es->setPayTarget(0);
	es->setName(esName);
	es->setBgColor(bgColor);

	QDomElement f = e.firstChildElement();
	while (!f.isNull()) {
		if (f.nodeName() == "group") {
			es->setGroup(getGroup(f.text().toInt()));
			CapiServerGroup* g = (CapiServerGroup*) es->getGroup();
			g->addEstate(es);
			es->setHousePrice(g->getHousePrice());
			es->setSellHousePrice(g->getHousePrice()/2);
			es->setColor(g->getColor());
			if (g->getBgColor() != "") es->setBgColor(g->getBgColor());
			if (g->getPrice() >= 0) es->setPrice(g->getPrice());
			es->setBuyable(es->getPrice() > 0);
		}
		else if (f.nodeName() == "icon") es->setPicture(f.text());
		else if (f.nodeName() == "price") {
			es->setPrice(f.text().toInt());
			es->setBuyable(es->getPrice() > 0);
		}
		else if (f.nodeName() == "tax") es->setTax(f.text().toInt());
		else if (f.nodeName() == "taxpercentage") es->setTaxPercent(f.text().toInt());
		else if (f.nodeName() == "paytarget") es->setPayTarget(getServerEstate(f.text().toInt(), true));
		else if (f.nodeName() == "takecard") es->setTakeCard(f.text());
		else if (f.nodeName() == "passmoney") es->setPassmoney(f.text().toInt());
		else if (f.nodeName() == "tojail") es->setToJail(f.text() == "1");
		else if (f.nodeName() == "jail") es->setJail(f.text() == "1");
		else if (f.nodeName() == "payamount") es->setPayAmount(f.text().toInt());
		else if (f.nodeName() == "freeparking") es->setFreeParking(f.text() == "1");
		else if (f.nodeName() == "go") es->setGo(f.text() == "1");
		else if (f.nodeName() == "bgcolor") es->setBgColor(f.text());
		else if (f.nodeName() == "advancetoifowned") es->setAdvanceToIfOwned(f.text().toInt());
		else if (f.nodeName() == "rent0") es->setRent(0, f.text().toInt());
		else if (f.nodeName() == "rent1") es->setRent(1, f.text().toInt());
		else if (f.nodeName() == "rent2") es->setRent(2, f.text().toInt());
		else if (f.nodeName() == "rent3") es->setRent(3, f.text().toInt());
		else if (f.nodeName() == "rent4") es->setRent(4, f.text().toInt());
		else if (f.nodeName() == "rent5") es->setRent(5, f.text().toInt());

		else qDebug() << "Unknon Estate property: " << f.nodeName() << "\n";

		f = f.nextSiblingElement();
	}
}

QString CapiServerGame::getConfigUpdate(int cid) {
	QString val = "";
	QString ret = "";
	QString base = "<configupdate configid=\"%1\" description=\"%4\" type=\"%2\" value=\"%3\" edit=\"1\" />";

	if  ((cid == 1) || (cid == -1))  ret += base.arg(1).arg("bool").arg(getCollectFine()).arg("Free Parking collects fine");
	if  ((cid == 2) || (cid == -1))  ret += base.arg(2).arg("bool").arg(getShuffleCards()).arg("Shuffle Cards before taking");
	if  ((cid == 3) || (cid == -1))  ret += base.arg(3).arg("bool").arg(getAuctions()).arg("Enable auctions");
	if  ((cid == 4) || (cid == -1))  ret += base.arg(4).arg("bool").arg(getDoubleMoneyOnExactLanding()).arg("Double passmoney on exact landing");
	if  ((cid == 5) || (cid == -1))  ret += base.arg(5).arg("bool").arg(getUnlimitedHouses()).arg("Unlimited houses/hotels");
	if  ((cid == 6) || (cid == -1))  ret += base.arg(6).arg("bool").arg(getJailNoRent()).arg("Players in jail gets no rent");
	if  ((cid == 7) || (cid == -1))  ret += base.arg(7).arg("bool").arg(getSellEnabled()).arg("Allow selling estates");
	if  ((cid == 8) || (cid == -1))  ret += base.arg(8).arg("bool").arg(getAutoTax()).arg("Automate tax decisions");
	if  ((cid == 9) || (cid == -1))  ret += base.arg(9).arg("bool").arg(getSpectatorsAllowed()).arg("Allow spectators");
	if ((cid == 10) || (cid == -1)) ret += base.arg(10).arg("int").arg(getStartMoney()).arg("Starting Cash");

	return ret;
}

void CapiServerGame::setConfig(int id, QString val) {
	int iVal = 0;
	bool ok;

	switch (id) {
	case 1:
		if (val == "1") collectFine = true;
		if (val == "0") collectFine = false;
		break;
	case 2:
		if (val == "1") shuffleCards = true;
		if (val == "0") shuffleCards = false;
		break;
	case 3:
		if (val == "1") auctions = true;
		if (val == "0") auctions = false;
		break;
	case 4:
		if (val == "1") doubleMoneyOnExactLanding = true;
		if (val == "0") doubleMoneyOnExactLanding = false;
		break;
	case 5:
		if (val == "1") unlimitedHouses = true;
		if (val == "0") unlimitedHouses = false;
		break;
	case 6:
		if (val == "1") jailNoRent = true;
		if (val == "0") jailNoRent = false;
		break;
	case 7:
		if (val == "1") sellEnabled = true;
		if (val == "0") sellEnabled = false;
		break;
	case 8:
		if (val == "1") autoTax = true;
		if (val == "0") autoTax = false;
		break;
	case 9:
		if (val == "1") spectatorsAllowed = true;
		if (val == "0") spectatorsAllowed = false;
		break;
	case 10:
		iVal = val.toInt(&ok);
		if (ok) {
			if (iVal > 0) startmoney = iVal;
		}
		break;
	default:
		qDebug(QString("No such config ID: "+QString::number(id)+"\n").toUtf8().data());
		break;
	}
}

QString CapiServerGame::removePlayerFromGame(CapiServerPlayer* p) {
	QString ret = "";

	//remove players estate owns
	foreach(CapiEstate* e, estates) {
		if (e->getOwner() == p) {
			CapiServerEstate* f = (CapiServerEstate*) e;
			f->setOwner(0);
			f->setHouses(0);
			f->setMortaged(false);
			setCanBuyOrSellHouses((CapiServerGroup*) f->getGroup());
			ret += f->getUpdate("owner houses mortgaged");
		}
	}

	//remove all player cards owns
	foreach(CapiCard* c, cards) {
		if (c->getOwner() == p) {
			CapiServerCard* d = (CapiServerCard*) c;
			d->setOwner(0);
			ret += d->getUpdate();
		}
	}

	p->setMoney(0);
	p->setBankrupt(false);
	p->setInJail(false);
	p->setCanRoll(false);
	ret += p->getUpdate("money bankrupt jailed can_roll");
	removePlayer(p);

	if ((getStatus() != "end") && (p == playerOnTurn)) {
		endTurn();
	}

	//Set new Master, if needed
	if (getMaster() != 0) {
		if (getMaster() == p) {
			if (players.count() > 0) {
				setMaster(players.value(0));
				ret += getUpdate("master");
			} else {
				setMaster(0);
			}
		}
	} else {
		qDebug() << "Bug: Game has no master.";
	}

	return ret;
}

QString CapiServerGame::getFullStatus() {
	QString ret = getUpdate();

	//Collect full Player updates
	foreach (CapiPlayer* p, players) {
		ret += ((CapiServerPlayer* ) p)->getUpdate();
	}

	//collect full estategroup-updates
	foreach (CapiGroup* g, groups) {
		ret += QString("<estategroupupdate groupid=\"%1\" name=\"%2\" />").arg(g->getId()).arg(g->getName());
	}

	//collect full estate updates
	foreach (CapiEstate* e, estates) {
		ret += ((CapiServerEstate* ) e)->getUpdate();
	}

	//collect full Card updates
	foreach (CapiCard* c, cards) {
		ret += ((CapiServerCard* ) c)->getUpdate();
	}

	return ret;
}

void CapiServerGame::startGame() {
	setStatus("init");
	foreach(CapiPlayer* p, players) {
		CapiServerPlayer* q = (CapiServerPlayer*) p;
		q->setMoney(getStartMoney());
		q->setMoved(true);
		q->setHasTurn(false);
		q->setCanRoll(false);
		q->setBankrupt(false);
		q->setInJail(false);
		q->setPos(goId);
	}

	//Mix up player order and cards
	for (int s = 0; s < 2000; s++) {
		//Players
		int i = rand()%players.size();
		int j = rand()%players.size();
		players.swap(i,j);

		//Cards
		i = rand()%cards.size();
		j = rand()%cards.size();
		cards.swap(i,j);
	}

	playerOnTurn = (CapiServerPlayer*) players.value(0);
	playerNum = 0;
	playerOnTurn->setHasTurn(true);
	playerOnTurn->setCanRoll(true);
	if (unlimitedHouses) {
		maxHouses = 999999999;
		maxHotels = 999999999;
	}
	sendToAll(getFullStatus());
	setStatus("run");
	sendToAll(getUpdate("status"));
	sendToAll(createDisplayMsg("Game started.", -1, true, true));
	sendToAll(createDisplayMsg(QString("Turn goes to %1.").arg(playerOnTurn->getName()), -1));
	sendToAll(createDisplayMsg("Warning: This Server is not marked as stable. It is possible that the server will crash, or do some silly things. Don't use it for official Tournaments. And NEVER let your Nuclear Weapons depend on this buggy server!", -1));
	turnState = 0;
}

void CapiServerGame::command(QString command, int pid) {
	CapiServerPlayer* p = (CapiServerPlayer*) getPlayer(pid, false);
	if (p == 0) return;
	if (command == ".r") {
		//Roll command
		if (p->getCanRoll() && (turnState == 0)) {

			p->setCanRoll(false);
			roll();
			QString txt = QString("%1 rolls %2 and %3.").arg(p->getName()).arg(r1).arg(r2);
			sendToAll(createDisplayMsg(txt, p->getPos(), true));

			//Caluclate new postion
			int newpos = (p->getPos()+r1+r2) % estates.count();

			//Check for doubles
			if (r1 == r2) p->incDoubles();

			if (p->getDoubles() == 3) {
				sendToAll(createDisplayMsg(QString("%1 has 3 doubles in row.").arg(playerOnTurn->getName()), p->getPos()));
				jailPlayer();
				endTurn();
			} else {
				//Notice all clients to move the player
				oldPos = p->getPos();
				p->setPos(newpos);
				waitForMove();
			}
		} else {
			p->sendMsg("You cannot roll at this moment.", "error");
		}
	} else if (command.startsWith(".t")) {
		//Turn position command
		if (turnState == 3) {
			bool ok;
			int pos = command.remove(0,2).toInt(&ok);
			if (ok) {
				CapiServerEstate* e = getServerEstate(pos);

				//Send Client a notice that the player gets money
				if (e->getPassmoney() > 0) {
				}

				//Notice Client that players lands
				if (pos == playerOnTurn->getPos()) {
					p->setMoved(true);
				}

				if (allPlayersMoved()) {
					//All clients have moved
					if (!directmove) {
						//Get all passmoneys
						int ps = oldPos;
						while (ps != playerOnTurn->getPos()) {
							ps = (ps+1)%estates.count();
							CapiServerEstate* es = getServerEstate(ps);
							if (es->getPassmoney() > 0) {
								playerOnTurn->setMoney(playerOnTurn->getMoney()+es->getPassmoney());
								sendToAll(createDisplayMsg(QString("%1 passes %2 and gets %3.").arg(playerOnTurn->getName(), es->getName()).arg(es->getPassmoney()), pos));
								sendToAll(playerOnTurn->getUpdate("money"));
							}
						}

						//Player gets double money on exact landing, if enabled
						if (doubleMoneyOnExactLanding && (e->getPassmoney() > 0)) {
							playerOnTurn->setMoney(playerOnTurn->getMoney()+e->getPassmoney());
							sendToAll(createDisplayMsg(QString("%1 gets %3 for landing on %2.").arg(playerOnTurn->getName(), e->getName()).arg(e->getPassmoney()), pos));
							sendToAll(playerOnTurn->getUpdate("money"));
						}
					}
					sendToAll(createDisplayMsg(QString("%1 lands on %2.").arg(playerOnTurn->getName(), e->getName()), pos));

					if ((e->getOwner() == 0) && e->getBuyable()) {
						waitForChoice();
					} else {
						handleEstate();
					}
				}
			}
		}
	} else if (command == ".jr") {
		//Roll in jail
		if (p->getHasTurn() && p->getInJail() && (turnState == 1)) {
			roll();

			if (r1 == r2) {
				unjailPlayer();
			} else {
				p->incJailRolls();
				sendToAll(createDisplayMsg(QString("No doubles. Stay in jail"), p->getPos(), true));
				if (p->getJailRolls() == 3) {
					payJail();
				} else {
					endTurn();
				}
			}
		}
	} else if (command == ".jp") {
		//pay for jail
		if (p->getHasTurn() && p->getInJail() && (turnState == 1)) {
			payJail();
		}
	} else if (command == ".jc") {
		//Use card
		if (p->getHasTurn() && p->getId() && (turnState == 1)) {
			//check if plyaer owns a jailcard
			//---search for one
			CapiServerCard* c = 0;
			foreach (CapiCard* d, cards) {
				CapiServerCard* e = (CapiServerCard*) d;
				if ((e->getOwner() == p) && e->getOutOfJail()) {
					c = e;
				}
			}
			if (c != 0) {
				//found one
				c->setOwner(0);
				p->setInJail(false);
				sendToAll(c->getUpdate()+p->getUpdate("jailed"));
				waitForRoll();
			}
		}
	} else if (command == ".eb") {
		//Buy estate
		if ((turnState == 4) && p->getHasTurn() && getEstate(p->getPos())->getBuyable() && (getEstate(p->getPos())->getOwner() == 0)) {
			CapiServerEstate* e = getServerEstate(p->getPos());
			if (p->getMoney() >= e->getPrice()) {
				e->setOwner(p);
				setCanBuyOrSellHouses((CapiServerGroup*) e->getGroup());
				e->setCanToggleMortage(true);
				p->setMoney(p->getMoney()-e->getPrice());
				p->setCanBuyEstate(false);
				p->setCanAuction(false);
				QString dsp = createDisplayMsg(QString("%1 buys %2").arg(p->getName(), e->getName()), e->getId());
				sendToAll(dsp+p->getUpdate("money can_buyestate canauction")+e->getUpdate("owner can_toggle_mortgage"));
				wasDouble();
			} else {
				p->sendMsg(QString("You don't have enough money to buy %1.").arg(e->getName()), "error");
			}
		}
	} else if (command == ".ea") {
		if ((turnState == 4) && p->getHasTurn() && getEstate(p->getPos())->getBuyable() && (getEstate(p->getPos())->getOwner() == 0)) {
			p->setCanBuyEstate(false);
			p->setCanAuction(false);
			sendToAll(p->getUpdate("can_buyestate canauction"));
			QString dsp = createDisplayMsg(QString("%1 chooses to auction %2").arg(p->getName(), getEstate(p->getPos())->getName()), p->getPos());
			sendToAll(dsp);
			auction();
		}
	} else if (command.startsWith(".ab")) {
		if (turnState == 5) {
			QStringList list = command.remove(0,3).split(":");
			if (list.size() >= 2) {
				bool ok1, ok2;
				int aid = ((QString) list.value(0)).toInt(&ok1);
				int bid = ((QString) list.value(1)).toInt(&ok2);

				if (ok1 && ok2) {
					if (aid == auctionid) {
						auctionBid(p, bid);
					} else {
						p->sendMsg("Invalid auctionid", "error");
					}
				} else {
					p->sendMsg("Invalid parameters.", "error");
				}
			} else {
				p->sendMsg("You need two parameters.", "error");
			}
		} else {
			p->sendMsg("Currently there is no auction", "error");
		}
	} else if (command == ".E") {
		if ((turnState == 4) && p->getHasTurn() && getEstate(p->getPos())->getBuyable() && (getEstate(p->getPos())->getOwner() == 0)) {
			p->setCanBuyEstate(false);
			p->setCanAuction(false);
			sendToAll(p->getUpdate("canauction can_buyestate"));
			wasDouble();
		}
	} else if (command.startsWith(".es")) {
		if (sellEnabled) {
			bool ok = false;
			int eid = command.remove(0,3).toInt(&ok);
			if (ok) {
				CapiServerEstate* e = (CapiServerEstate*) getEstate(eid, false);
				if (e != 0) {
					if (e->getOwner() == p) {
						if (e->getGroup()->getNumHouses() == 0) {
							if (!e->getMortaged()) {
								e->setOwner(0);
								p->setMoney(p->getMoney()+e->getPrice()*0.75);
								sendToAll(e->getUpdate("owner")+p->getUpdate("money"));
								setCanBuyOrSellHouses((CapiServerGroup*) e->getGroup());
								checkForPayment();
							} else {
								p->sendMsg("You cannot sell mortgaged estates", "error");
							}
						} else {
							p->sendMsg("You cannot sell an estates with houses on its group", "error");
						}
					} else {
						p->sendMsg(QString("You do not own %1").arg(e->getName()), "error");
					}
				} else {
					p->sendMsg("Estate not found.", "error");
				}
			} else {
				p->sendMsg("Could not parse estateid", "error");
			}
		} else {
			p->sendMsg("Selling is not allowed in this game.", "error");
		}
	} else if (command == ".T$") {
		if (p->getHasTurn() && (turnState == 6) && (taxamount != 0)) {
			sendToAll(createDisplayMsg(QString("%1 chooses to pay %2").arg(p->getName()).arg(taxamount), p->getPos()));
			CapiEstate* targetEstate = getServerEstate(p->getPos())->getPayTarget();
			depts.append(new CapiDept(p, 0, targetEstate, taxamount));
			waitForDept();
		}
	} else if (command == ".T%") {
		if (p->getHasTurn() && (turnState == 6) && (taxrate != 0)) {
			sendToAll(createDisplayMsg(QString("%1 chooses to pay %2%").arg(p->getName()).arg(taxrate), p->getPos()));
			int amount = (taxrate*assetsForPlayer(playerOnTurn))/100;
			CapiEstate* targetEstate = getServerEstate(p->getPos())->getPayTarget();
			depts.append(new CapiDept(p, 0, targetEstate, amount));
			waitForDept();
		}
	} else if (command.startsWith(".em")) {
		//(Un)Mortgage estate
		if (turnState != 3) {
			//Only allowed when players are not moving
			bool ok;

			int eid = command.remove(0,3).toInt(&ok);
			if (ok) {
				CapiServerEstate* e = getServerEstate(eid);

				//Only owner can toggle mortgage
				if (e->getOwner() == p) {
					int mprice = e->getUnmortageVal();
					if (!e->getMortaged()) mprice = (-1)*e->getMortageVal();

					if ((mprice < 0) || (p->getMoney() > mprice)) {
						if (e->getCanToggleMortage()) {
							e->setMortaged(!e->getMortaged());
							setCanBuyOrSellHouses((CapiServerGroup*) e->getGroup());
							p->setMoney(p->getMoney()-mprice);
							QString msg;
							if (e->getMortaged())
								msg = createDisplayMsg(QString("%1 mortgaged %2.").arg(p->getName(), e->getName()), playerOnTurn->getPos());
							else
								msg = createDisplayMsg(QString("%1 unmortgaged %2.").arg(p->getName(), e->getName()), playerOnTurn->getPos());
							sendToAll(msg+p->getUpdate("money")+e->getUpdate("mortgaged"));

							checkForPayment();
						} else {
							p->sendMsg(QString("You cannot toggle mortgage of %1").arg(e->getName()), "error");
						}
					} else {
						p->sendMsg(QString("You need %1 to unmortgage %2 it.").arg(mprice).arg(e->getName()), "error");
					}
				} else {
					p->sendMsg(QString("Sorry: You doon't own %1. It is unfair to mortgaged other players estates.").arg(e->getName()), "error");
				}
			} else {
				p->sendMsg("Invalid Estate-ID", "error");
			}
		} else {
			p->sendMsg("You cannot toggle mortgage while players are moving.", "error");
		}
	} else if (command.startsWith(".hb")) {
		//Buy house on estate
		if (turnState != 3) {
			bool ok;
			int eid = command.remove(0,3).toInt(&ok);
			if (ok) {
				CapiServerEstate* e = getServerEstate(eid);
				if ((e->getOwner() == p) && e->getCanBuyHouse()) {
					if (e->getHousePrice() <= p->getMoney()) {
						//Check if we may buy a house/hotel
						int houses = getNumHouses();
						int hotels = getNumHotels();
						if (((hotels < maxHotels) && (e->getHouses() == 4)) ||
						    ((houses < maxHouses) && (e->getHouses()  < 4))) {
							e->setHouses(e->getHouses()+1);

							setCanBuyOrSellHouses((CapiServerGroup*) e->getGroup());
							p->setMoney(p->getMoney()-e->getHousePrice());
							sendToAll(p->getUpdate("money")+e->getUpdate("houses"));
							sendToAll(createDisplayMsg(QString("%1 buys a house on %2").arg(p->getName(), e->getName()), playerOnTurn->getPos()));
							checkForPayment();
						} else {
							if (e->getHouses() < 4)
								p->sendMsg(QString("Cannot buy a house. All houses are selled."), "error");
							else
								p->sendMsg(QString("Cannot buy a hotel. All hotels are selled."), "error");
						}
					} else {
						p->sendMsg(QString("You need %1 to build a house on %2").arg(e->getHousePrice()).arg(e->getName()), "error");
					}
				} else {
					p->sendMsg(QString("You cannot build a house on %1. Is it really your estate?").arg(e->getName()), "error");
				}
			} else {
				p->sendMsg("Invalid Estate-ID", "error");
			}
		} else {
			p->sendMsg("Patien: Wait until the movements are finished.", "error");
		}
	} else if (command.startsWith(".hrb")) {
		//Buy a houserow on group
		if (turnState != 3) {
			bool ok;
			int gid = command.remove(0,4).toInt(&ok);
			if (ok) {
				CapiServerGroup* g = (CapiServerGroup*) getGroup(gid);
				int maxH = 0;
				for (int i = 0; i < g->getNumEstates(); i++)
					if (g->getEstate(i)->getHouses() > maxH) maxH = g->getEstate(i)->getHouses();
				if (g->getAllEstatesSameOwner() && (g->getEstate(0)->getOwner() == p) && (maxH < 5)) {
					int neededMoney = 0;
					for (int i = 0; i < g->getNumEstates(); i++)
						neededMoney += g->getEstate(i)->getHousePrice();
					if (neededMoney <= p->getMoney()) {
						for (int i = 0; i < g->getNumEstates(); i++) {
							CapiServerEstate* e = (CapiServerEstate*) g->getEstate(i);
							e->setHouses(e->getHouses()+1);
							p->setMoney(p->getMoney()-e->getHousePrice());
							sendToAll(p->getUpdate("money")+e->getUpdate("houses"));
						}
						setCanBuyOrSellHouses((CapiServerGroup*) g);
						sendToAll(createDisplayMsg(QString("%1 buys a houserow on group %2").arg(p->getName(), g->getName()), playerOnTurn->getPos()));
						checkForPayment();
					} else {
						p->sendMsg(QString("You need %1 to build a houserwo on %2").arg(neededMoney).arg(g->getName()), "error");
					}
				} else {
					p->sendMsg(QString("You cannot build a houserow on %1").arg(g->getName()), "error");
				}
			} else {
				p->sendMsg("Invalid Group-ID", "error");
			}
		} else {
			p->sendMsg("You cannot buy houserows players are moving.", "error");
		}
	} else if (command.startsWith(".hs")) {
		//Sell house on estate
		if (turnState != 3) {
			bool ok;
			int eid = command.remove(0,3).toInt(&ok);
			if (ok) {
				CapiServerEstate* e = getServerEstate(eid);
				if ((e->getOwner() == p) && e->getCanSellHouse()) {
					e->setHouses(e->getHouses()-1);
					setCanBuyOrSellHouses((CapiServerGroup*) e->getGroup());
					p->setMoney(p->getMoney()+e->getSellHousePrice());
					sendToAll(p->getUpdate("money")+e->getUpdate("houses"));
					sendToAll(createDisplayMsg(QString("%1 sells a house on %2").arg(p->getName(), e->getName()), playerOnTurn->getPos()));
					checkForPayment();
				} else {
					p->sendMsg(QString("You cannot sell a house on %1").arg(e->getName()), "error");
				}
			} else {
				p->sendMsg("Invalid Estate-ID", "error");
			}
		} else {
			p->sendMsg("You cannot sell houses players are moving.", "error");
		}
	} else if (command.startsWith(".Tn")) {
		bool ok;
		int tpid = command.remove(0,3).toInt(&ok);
		if (ok) {
			CapiServerPlayer* tradep = (CapiServerPlayer*) getPlayer(tpid, false);
			if (tradep != 0) {
				if (tradep != p) {
					CapiTrade* t = new CapiTrade(tradeid++, p, tradep);
					trades.append(t);
				} else {
					p->sendMsg("Yes! Trading with yourself makes sense. But you don't need me to do this.", "info");
				}
			} else {
				p->sendMsg(QString("Cannont find Player with ID %1").arg(tpid), "error");
			}
		} else {
			p->sendMsg("Invalid PlayerID", "error");
		}
	} else if (command.startsWith(".Tr")) {
		bool ok;
		int tid = command.remove(0,3).toInt(&ok);
		if (ok) {
			//Find the trade
			CapiTrade* t = 0;
			foreach(CapiTrade* u, trades) if (u->getId() == tid) t = u;

			if (t != 0) {
				if (t->playerInvolved(p)) {
					t->reject();
					trades.removeAll(t);
					delete t;
				} else {
					p->sendMsg("Nice try.But you should not stop a trade, that you cannot know", "error");
				}
			} else {
				p->sendMsg(QString("Trade with ID %1 not found").arg(tid), "error");
			}
		} else {
			p->sendMsg("Invalid TradeID", "error");
		}
	} else if (command.startsWith(".Ta")) {
		QStringList list = command.remove(0,3).split(":");
		if (list.size() == 2) {
			bool ok1, ok2;
			int tid = ((QString) list.value(0)).toInt(&ok1);
			int trev = ((QString) list.value(1)).toInt(&ok2);

			if (ok1 && ok2) {
				//Find the trade
				CapiTrade* t = 0;
				foreach(CapiTrade* u, trades) if (u->getId() == tid) t = u;

				if (t != 0) {
					if (t->playerInvolved(p)) {
						t->accept(p, trev);
						if (t->checkForAccepts()) {
							sendToAll(t->run());
							trades.removeAll(t);
							delete t;
							checkForPayment();
							foreach (CapiGroup* g, groups)
								setCanBuyOrSellHouses((CapiServerGroup*) g);
						}
					} else {
						p->sendMsg("Nice try.But you should not accept Trades that you cannot know", "error");
					}
				} else {
					p->sendMsg("Could not find the trade", "error");
				}
			} else {
				p->sendMsg("Invalid TradeID or Revision.", "error");
			}
			}
	} else if (command.startsWith(".Te")) {
		QStringList list = command.remove(0,3).split(":");
		if (list.size() == 3) {
			bool ok1, ok2, ok3;
			int tid = ((QString) list.value(0)).toInt(&ok1);
			int teid = ((QString) list.value(1)).toInt(&ok2);
			int tpid = ((QString) list.value(2)).toInt(&ok3);

			if (ok1 && ok2 && ok3) {
				//Find the trade
				CapiTrade* t = 0;
				foreach(CapiTrade* u, trades) if (u->getId() == tid) t = u;
				if (t != 0) {
					//Found trade
					if (t->playerInvolved(p)) {
						//Player is involved
						CapiEstate* e = getEstate(teid, false);
						if (e != 0) {
							CapiPlayer* tp = getPlayer(tpid, false);
							t->updateEstateTrade(tp, e);
						} else {
							p->sendMsg("Unknown estateID", "error");
						}
					} else {
						p->sendMsg("You cannot change trade condition of trades that you cannot know", "error");
					}
				} else {
					p->sendMsg("Could not find the trade", "error");
				}
			} else {
				p->sendMsg("Invalid TradeID, estateID or playerID.", "error");
			}
		} else {
			p->sendMsg("Incorrect number of parameters", "error");
		}
	} else if (command.startsWith(".Tc")) {
		QStringList list = command.remove(0,3).split(":");
		if (list.size() == 3) {
			bool ok1, ok2, ok3;
			int tid = ((QString) list.value(0)).toInt(&ok1);
			int tcid = ((QString) list.value(1)).toInt(&ok2);
			int tpid = ((QString) list.value(2)).toInt(&ok3);

			if (ok1 && ok2 && ok3) {
				//Find the trade
				CapiTrade* t = 0;
				foreach(CapiTrade* u, trades) if (u->getId() == tid) t = u;
				if (t != 0) {
					//Found trade
					if (t->playerInvolved(p)) {
						//Player is involved
						CapiCard* c = getCard(tcid, false);
						if (c != 0) {
							CapiPlayer* tp = getPlayer(tpid, false);
							t->updateCardTrade(tp, c);
						} else {
							p->sendMsg("Unknown cardID", "error");
						}
					} else {
						p->sendMsg("You cannot change conditions of a trade that you cannot know", "error");
					}
				} else {
					p->sendMsg("Could not find the trade", "error");
				}
			} else {
				p->sendMsg("Invalid TradeID, cardID or playerID.", "error");
			}
		} else {
			p->sendMsg("Incorrect number of parameters", "error");
		}
	} else if (command.startsWith(".Tm")) {
		QStringList list = command.remove(0,3).split(":");
		if (list.size() == 4) {
			bool ok1, ok2, ok3, ok4;
			int tid =  ((QString) list.value(0)).toInt(&ok1);
			int tfid = ((QString) list.value(1)).toInt(&ok2);
			int ttid = ((QString) list.value(2)).toInt(&ok3);
			int tm =   ((QString) list.value(3)).toInt(&ok4);

			if (ok1 && ok2 && ok3 && ok4) {
				//Find the trade
				CapiTrade* t = 0;
				foreach(CapiTrade* u, trades) if (u->getId() == tid) t = u;
				if (t != 0) {
					//Found trade
					if (t->playerInvolved(p)) {
						//Player is involved
						CapiPlayer* tp = getPlayer(ttid, false);
						CapiPlayer* fp = getPlayer(tfid, false);

						t->updateMoneyTrade(fp, tp, tm);
					} else {
						p->sendMsg("You cannot change trade condition of trades that you cannot know", "error");
					}
				} else {
					p->sendMsg("Could not find the trade", "error");
				}
			} else {
				p->sendMsg("Invalid TradeID, fromPlayerID or toPlayerID.", "error");
			}
		} else {
			p->sendMsg("Incorrect number of parameters", "error");
		}
	} else if (command==".cheat") {
		//A small cheat for debugging
		p->setMoney(10000);
		p->setName(p->getName()+" (Damn Cheater!)");
		sendToAll(p->getUpdate("money name"));
		sendToAll(createDisplayMsg(QString("%1 cheated!!!").arg(p->getName()), playerOnTurn->getPos()));
		checkForPayment();
	} else {
			p->sendMsg("Unknown command.", "error");
	}
}

QString CapiServerGame::createButton(QString cmd, QString text) {
	return QString("<button command=\"%1\" caption=\"%2\" enabled=\"1\" />").arg(maskXML(cmd), maskXML(text));
}

QString CapiServerGame::createDisplayMsg(QString text, int estateid, bool clearText, bool clearButtons, QString buttons) {
	QString ret;
	ret = QString("<display estateid=\"%1\" text=\"%2\" cleartext=\"%3\" clearbuttons=\"%4\">").arg(estateid).arg(maskXML(text)).arg(clearText).arg(clearButtons);
	ret += buttons;
	ret += "</display>";
	
	return ret;
}

void CapiServerGame::sendToAll(QString msg, int exclude) {
	foreach(CapiPlayer* p, players) {
		CapiServerPlayer* q = (CapiServerPlayer*) p;
		if (q->getId() != exclude) q->send(msg);
	}
}

bool CapiServerGame::allPlayersMoved() {
	foreach(CapiPlayer* p, players) {
		CapiServerPlayer* q = (CapiServerPlayer*) p;
		if (!q->getMoved()) return false;
	}
	return true;
}

bool CapiServerGame::playerOwnsOutOfJailCard(CapiServerPlayer* p) {
	foreach (CapiCard* d, cards) {
		CapiServerCard* c = (CapiServerCard*) d;
		if (c->getOutOfJail() && (c->getOwner() == p)) return true;
	}
	return false;
}

void CapiServerGame::roll() {
	r1 = (rand() % 6) + 1;
	r2 = (rand() % 6) + 1;
}

void CapiServerGame::waitForRoll() {
	turnState = 0;
	playerOnTurn->setCanRoll(true);
	sendToAll(playerOnTurn->getUpdate("hasturn can_roll"));
}

void CapiServerGame::waitForJail() {
	turnState = 1;
	playerOnTurn->setCanRoll(false);
	playerOnTurn->setCanUseCard(playerOwnsOutOfJailCard(playerOnTurn));
	QString buttons = createButton(".jr", "Roll");
	if (playerOnTurn->getCanUseCard()) buttons += createButton(".jc", "Use card");
	buttons += createButton(".jp", "Pay");
	sendToAll(playerOnTurn->getUpdate("hasturn can_roll canusecard"));
	sendToAll(createDisplayMsg(QString("%1 ist in jail. Round: %2").arg(playerOnTurn->getName()).arg(playerOnTurn->getJailRolls()+1), playerOnTurn->getPos(), false, false), playerOnTurn->getId());
	playerOnTurn->send(createDisplayMsg(QString("%1 ist in jail. Round: %2").arg(playerOnTurn->getName()).arg(playerOnTurn->getJailRolls()+1), playerOnTurn->getPos(), false, false, buttons));
}

void CapiServerGame::payJail() {
	turnState = 2;
	//Pay the money
	CapiServerEstate* e = getServerEstate(playerOnTurn->getPos());
	if (playerOnTurn->getMoney() >= e->getPayAmount()) {
		playerOnTurn->setMoney(playerOnTurn->getMoney()-e->getPayAmount());
		playerOnTurn->setInJail(false);
		if (e->getPayTarget() != 0) {
			CapiServerEstate* t = (CapiServerEstate*) e->getPayTarget();
			t->setMoney(t->getMoney()+e->getPayAmount());
			sendToAll(t->getUpdate("money"));
		}

		sendToAll(createDisplayMsg(QString("%1 has payed %2 an now leaves jail").arg(playerOnTurn->getName()).arg(e->getPayAmount()), playerOnTurn->getPos()));
		sendToAll(playerOnTurn->getUpdate("money jailed"));
		waitForRoll();
	} else {
		sendToAll(createDisplayMsg(QString("%1 must pay %2. He/he must raise his/her cash.").arg(playerOnTurn->getName()).arg(e->getPayAmount()), playerOnTurn->getPos()));
	}
}

void CapiServerGame::waitForMove() {
	turnState = 3;
	//Notice all clients to move the player
	foreach (CapiPlayer* p, players) {
		((CapiServerPlayer*) p)->setMoved(false);
		p->setDirectmove(directmove);
	}
	sendToAll(playerOnTurn->getUpdate("location can_roll directmove"));
}

void CapiServerGame::waitForChoice() {
	turnState = 4;
	CapiServerEstate* e = getServerEstate(playerOnTurn->getPos());
	QString buttons = createButton(".eb", "Buy");
	if (auctions) buttons += createButton(".ea", "Auction");
	else buttons += createButton(".E", "End Turn");
	sendToAll(createDisplayMsg(QString("%1 can buy %2.").arg(playerOnTurn->getName(), e->getName()), e->getId(), false, false), playerOnTurn->getId());
	playerOnTurn->send(createDisplayMsg(QString("%1 can buy %2.").arg(playerOnTurn->getName(), e->getName()), e->getId(), false, false, buttons));
	playerOnTurn->setCanAuction(auctions);
	playerOnTurn->setCanBuyEstate(true);

	sendToAll(playerOnTurn->getUpdate("canauction can_buyestate"));
}

void CapiServerGame::auction() {
	turnState = 5;
	//Begin a new auction
	auctionid = 1;
	auctionStatus = 0;
	auctionActor = playerOnTurn;
	auctionEstate = getServerEstate(playerOnTurn->getPos());
	auctionHighBidder = 0;
	auctionHighestBid = 0;
	statusCount = 0;

	sendToAll(auctionUpdate("actor, status, estateid"));
}

void CapiServerGame::auctionBid(CapiServerPlayer* bidder, int bid) {
	if (auctionid == -1) {
		bidder->sendMsg("Currently we have not an auction.", "error");
		return;
	}

	if (bid <= auctionHighestBid) {
		bidder->sendMsg(QString("Minimum bid is %1.").arg(auctionHighestBid), "error");
		return;
	}

	if (bid > assetsForPlayer(bidder)) {
		bidder->sendMsg("You can't bid more than you have.", "error");
		return;
	}
	if (auctionStatus == 3) {
		bidder->sendMsg("Too late for bids.", "error");
		return;
	}

	auctionHighestBid = bid;
	auctionHighBidder = bidder;
	auctionStatus = 0;

	sendToAll(auctionUpdate("highbid highbidder status"));
}

void CapiServerGame::waitForAuctionPay() {

	if (auctionid == -1) return;
	if (auctionStatus != 3) return;

	if (auctionHighBidder->getMoney() >= auctionHighestBid) {
		//Highest Bidder can pay the bid
		auctionEstate->setOwner(auctionHighBidder);
		auctionHighBidder->setMoney(auctionHighBidder->getMoney()-auctionHighestBid);
		sendToAll(auctionEstate->getUpdate("owner")+auctionHighBidder->getUpdate("money"));
		resetAuction();
		wasDouble();
		return;
	}

	if (auctionHighestBid > assetsForPlayer(auctionHighBidder)) {
		//Highest bidder can't pay it. Stop auction
		sendToAll(createDisplayMsg(QString("%1 was so stupid to waste enough money so that he/she can't pay %2 anymore. How the hell can you waste so much money. If you don't want to have %2, you shoulds not bid for it. Next time i'll let you go bankrupt!").arg(auctionHighBidder->getName(), auctionEstate->getName()), playerOnTurn->getPos()));
		resetAuction();
		wasDouble();
		return;
	}

	//We get here if player can pay hid bid, but must raise cash first
	sendToAll(createDisplayMsg(QString("%1 must raise cash to %2, to pay %3").arg(auctionHighBidder->getName()).arg(auctionHighestBid).arg(auctionEstate->getName()), playerOnTurn->getPos()));
}

void CapiServerGame::resetAuction() {
		//Reset auction data
		auctionState = 0;
		auctionid = -1;
		auctionStatus = 0;
		auctionActor = 0;
		auctionEstate = 0;
		auctionHighBidder = 0;
		auctionHighestBid = 0;
		statusCount = 0;
}

QString CapiServerGame::auctionUpdate(QString type) {
	if (auctionid == -1) return "";

	QString ret = "<auctionupdate ";
	ret += QString("auctionid=\"%1\" ").arg(auctionid);
	if ((type == "") || type.contains("actor"))      ret += QString("actor=\"%1\" ").arg(playerOnTurn->getId());
	if ((type == "") || type.contains("estateid"))   ret += QString("estateid=\"%1\" ").arg(auctionEstate->getId());
	if ((type == "") || type.contains("highbid"))    ret += QString("highbid=\"%1\" ").arg(auctionHighestBid);
	if ((type == "") || type.contains("highbidder")) ret += QString("highbidder=\"%1\" ").arg(auctionHighBidder->getId());
	if ((type == "") || type.contains("status"))     ret += QString("status=\"%1\" ").arg(auctionStatus);

	ret += " />";
	return ret;
}

void CapiServerGame::waitForTax(int ta, int tr) {
	turnState = 6;

	taxamount = ta;
	taxrate = tr;
}

void CapiServerGame::waitForDept() {
	turnState = 7;

	bool allPayed = true;
	foreach (CapiDept* d, depts) {
		if (d->getPayed()) continue;

		//Check if ower can pay the dept
		if (d->getAmount() > assetsForPlayer(d->getFromPlayer())) {
			//ower is not able to raise his cash to the dept-amount
			bankruptPlayer(d);
		} else {
			//owner can raise his cash, or pay the dept
			//Depts was not payed yet try to pay
			bool payed = d->pay();

			QString target = "the bank";
			if (d->getToPlayer() != 0) target = d->getToPlayer()->getName();
			if (!payed) {
				//ower can not pay the dept
				sendToAll(
				  createDisplayMsg(
				    QString("%1 owes %2 to %3 but is not solvent. %1 must raise cach to %2 first.").arg(d->getFromPlayer()->getName(), QString::number(d->getAmount()), target),
				    playerOnTurn->getPos()
				  )
				);
				allPayed = false;
			} else {
				//ower has paid the dept
				sendToAll(
				  createDisplayMsg(
				    QString("%1 pays %2 to %3.").arg(d->getFromPlayer()->getName(), QString::number(d->getAmount()), target),
				    playerOnTurn->getPos()
				  )
				);
				sendToAll(((CapiServerPlayer*) d->getFromPlayer())->getUpdate("money"));
				if (d->getToPlayer() != 0)
					sendToAll(((CapiServerPlayer*) d->getToPlayer())->getUpdate("money"));
				if (d->getToEstate() != 0)
					sendToAll(((CapiServerEstate*) d->getToEstate())->getUpdate("money"));
			}
		}
	}

	if (allPayed) {
		//If all depts payed delete all depts and continue game
		while (depts.size() > 0) {
			CapiDept* d = depts.value(0);
			depts.removeAll(d);
			delete d;
		}
		if (deptRound > 0)
			sendToAll(createDisplayMsg("All depts settled. Game continues", playerOnTurn->getPos()));
		deptRound = 0;
		movedPlayer();
	} else {
		deptRound++;
	}
}

void CapiServerGame::checkForPayment() {
		if (turnState == 2)
			payJail();

		if (turnState == 5)
			waitForAuctionPay();

		if (turnState == 7)
			waitForDept();
}

void CapiServerGame::jailPlayer() {
	playerOnTurn->setDirectmove(true);
	playerOnTurn->resetJailRolls();

	//search jail
	for (int i = 1; i <= estates.count(); i++) {
		int eid = playerOnTurn->getPos()-i;
		while (eid < 0) eid += estates.count();
		while (eid >= estates.count()) eid -= estates.count();
		CapiServerEstate* e = getServerEstate(eid);
		if (e->getJail()) {
			playerOnTurn->setPos(eid);
			break;
		}
	}

	playerOnTurn->setInJail(true);
	sendToAll(playerOnTurn->getUpdate("can_roll location jailed directmove"));
}

void CapiServerGame::unjailPlayer() {
	playerOnTurn->setInJail(false);
	playerOnTurn->setDirectmove(false);

	//Caluclate new postion
	int newpos = (playerOnTurn->getPos()+r1+r2) % estates.count();
	QString txt = QString("%1 rolls doubles and can leave jail").arg(playerOnTurn->getName());
	sendToAll(createDisplayMsg(txt, playerOnTurn->getPos()));
	oldPos = playerOnTurn->getPos();
	playerOnTurn->setPos(newpos);

	sendToAll(playerOnTurn->getUpdate("jailed directmove pos"));
	waitForMove();
}

void CapiServerGame::handleEstate() {
	bool wait = false;
	playerMoved = false;
	directmove = false;
	CapiServerEstate* e = getServerEstate(playerOnTurn->getPos());

	//GoToJailestate
	if (e->getToJail()) {
		jailPlayer();
	}

	//Free parking estate
	if (collectFine && e->getFreeParking() && (e->getMoney() > 0)) {
		sendToAll(createDisplayMsg(QString("%1 collects %2 from fine.").arg(playerOnTurn->getName()).arg(e->getMoney()), e->getId()));
		playerOnTurn->setMoney(playerOnTurn->getMoney()+e->getMoney());
		e->setMoney(0);
		sendToAll(playerOnTurn->getUpdate("money")+e->getUpdate("money"));
	}

	//Tax estate
	int ta = e->getTax();
	int tr = e->getTaxPercent();
	int options = 0;
	if (ta > 0) options++;
	if (tr > 0) options++;

	if (options == 2) {
		QString txt = QString("%1 must pay %2 or %3% of his/her assets").arg(playerOnTurn->getName()).arg(ta).arg(tr);
		QString buttons = createButton(".T$", QString("Pay %1").arg(ta));
		buttons += createButton(".T%", QString("Pay %1%").arg(tr));
		if (!autoTax) {
			sendToAll(createDisplayMsg(txt, e->getId(), false, false), playerOnTurn->getId());
			playerOnTurn->send(createDisplayMsg(txt, e->getId(), false, false, buttons));
			waitForTax(ta, tr);
			wait = true;
		} else {
			sendToAll(createDisplayMsg(txt, e->getId()));
			//If automate tax decition is active, get min tax rate
			int trrate = (tr*assetsForPlayer(playerOnTurn))/100;
			int tarate = ta;
			int tax = tarate;
			if (trrate < tarate) tax = trrate;

			depts.append(new CapiDept(playerOnTurn, 0, e->getPayTarget(), tax));
			waitForDept();
			wait = true;
		}
	} else if (options == 1) {
		int taxrate = (tr*assetsForPlayer(playerOnTurn))/100;
		taxrate += ta;
		QString txt = QString("%1 must pay %2").arg(playerOnTurn->getName()).arg(taxrate);
		sendToAll(createDisplayMsg(txt, e->getId()));

		depts.append(new CapiDept(playerOnTurn, 0, e->getPayTarget(), taxrate));
		waitForDept();
		wait = true;
	}

	//Handle rent
	if (e->getOwner() != 0) {
		CapiServerGroup* g = (CapiServerGroup*) e->getGroup();
		CapiServerPlayer* owner = (CapiServerPlayer*) e->getOwner();

		//Estate is owned by somebody
		if (owner == playerOnTurn) {
			//We land on our own estate
			sendToAll(createDisplayMsg(QString("%1 already owns it.").arg(playerOnTurn->getName()), e->getId()));
		} else {
			int rent = 0;
			if (!e->getMortaged()) {
				//calulate the rent for the estate in two steps:
				//first: normal rent
				//second: by rentmath

				//first normal rent
				rent = e->getRent(e->getHouses());
				if (e->getHouses() == 0) {
					//We must double rent for no houses, if owner own tha whole group
					CapiGroup* g = e->getGroup();
					if (g->getAllEstatesSameOwner()) {
						rent += rent;
					}
				}

				//second, calculate the rent my given math if needed. It will overwrite the normal-rent
				if (g->getRentMath() != "") {
					int dice = r1+r2;
					int groupown = 0;
					//Get the number of estates in the group, that is owned by owner
					for (int i = 0; i < g->getNumEstates(); i++)
						if (g->getEstate(i)->getOwner() == owner) groupown++;

					//Caluclate rent
					QString math = g->getRentMath();
					if (replaceMath != "") math = replaceMath;
					math = math.replace("${DICE}", QString::number(dice)).replace("${GROUPOWNED}", QString::number(groupown));
					QScriptEngine engine;
					rent = engine.evaluate(math).toInteger();
				}

				if (!(owner->getInJail() && jailNoRent)) {
					// playeronTurn has to pay the rent
					depts.append(new CapiDept(playerOnTurn, owner, 0, rent));
					wait = true;
					waitForDept();
				} else {
					sendToAll(createDisplayMsg(QString("%1 pays no rent, because %2 is in jail.").arg(playerOnTurn->getName(), owner->getName()), e->getId()));
				}
			} else {
				sendToAll(createDisplayMsg(QString("%1 pays no rent, because it is mortgaged.").arg(playerOnTurn->getName()), e->getId()));
			}
		}
	}
	replaceMath = "";

	//Handle takecard
	if (e->getTakeCard() != "") {
		qDebug() << "Needed cardgroup: " << e->getTakeCard();
		//Draw a card from stack
		//Find all cards that are not owned and are in the correct group
		QList<CapiCard*> cardList;
		for (int i = 0; i < cards.size(); i++) {
			CapiServerCard* d = (CapiServerCard*) cards.value(i);
			qDebug() << "Card #" << QString::number(d->getId()) << "is from group " << d->getGroup();
			if ((d->getGroup() == e->getTakeCard()) && (d->getOwner() == 0)) {
				cardList.append(d);
			}
		}

		if (cardList.size() == 0) {
			//We found to card
			sendToAll(createDisplayMsg("Found no card.", playerOnTurn->getPos()));
		} else {
			//We found some cards. take one
			int cardNum = 0;
			if (shuffleCards) cardNum = (rand() % cardList.size());
			CapiServerCard* c = (CapiServerCard*) cardList.value(cardNum);

			//Bring it at the buttom from the stack
			cards.removeAll(c);
			cards.append(c);

			//Now handle the Card
			sendToAll(createDisplayMsg(c->getText(), playerOnTurn->getPos()));

			//Simple advance
			if (c->getAdvance() > 0) {
				int newPos = (playerOnTurn->getPos()+c->getAdvance())%estates.size();
				playerOnTurn->setPos(newPos);
				playerMoved = true;
			} else if (c->getAdvance() < 0) {
				int newPos = playerOnTurn->getPos()+c->getAdvance();
				while (newPos < 0) newPos += estates.size();
				playerOnTurn->setPos(newPos);
				playerMoved = true;
				directmove = true;
			}

			//Advance to
			if (c->getAdvanceTo() != -1) {
				playerOnTurn->setPos(c->getAdvanceTo());
				playerMoved = true;
			}

			//Advance to next of
			if (c->getAdvanceToNextOf() != "") {
				int pos = (playerOnTurn->getPos()+1)%estates.size();
				while (pos != playerOnTurn->getPos()) {
					if ((getEstate(pos)->getGroup() != 0) && (getEstate(pos)->getGroup()->getName() == c->getAdvanceToNextOf())) {
						break;
					}
					pos = (pos+1)%estates.size();
				}
				playerMoved = (pos != playerOnTurn->getPos());
				playerOnTurn->setPos(pos);
			}

			//Ownership
			if (c->getCanBeOwned()) {
				c->setOwner(playerOnTurn);
				sendToAll(c->getUpdate());
			}

			//Player has to pay
			if (c->getPay() > 0) {
				depts.append(new CapiDept(playerOnTurn, 0, e->getPayTarget(), c->getPay()));
				wait = true;
				waitForDept();
			}

			//Player gets money
			if (c->getPay() < 0) {
				playerOnTurn->setMoney(playerOnTurn->getMoney()-c->getPay());
				sendToAll(playerOnTurn->getUpdate("money"));
			}

			//Player has to pay to each player
			if (c->getPayEach() > 0) {
				foreach(CapiPlayer* q, players) {
					if (q != playerOnTurn) {
						depts.append(new CapiDept(playerOnTurn, q, 0, c->getPayEach()));
					}
				}
				wait = true;
				waitForDept();
			}

			//Player gets money from each player
			if (c->getPayEach() < 0) {
				int amount = -1 * c->getPayEach();
				foreach(CapiPlayer* q, players) {
					if (q != playerOnTurn) {
						depts.append(new CapiDept(q, playerOnTurn, 0, amount));
					}
				}
				wait = true;
				waitForDept();
			}

			//Player has to pay for his houses/hotels
			if ((c->getPayHotel() > 0) || (c->getPayHouse() > 0)) {
				int amount = 0;
				amount += c->getPayHouse() * getNumHouses(playerOnTurn);
				amount += c->getPayHotel() * getNumHotels(playerOnTurn);

				depts.append(new CapiDept(playerOnTurn, 0, e->getPayTarget(), amount));
				wait = true;
				waitForDept();
			}

			if (c->getRentMath() != "") {
				replaceMath = c->getRentMath();
			}

			//Roll if needed
			if (c->getRoll()) {
				roll();
			}

			//Jail player if needed
			if (c->getToJail()) {
				jailPlayer();
			}
		}
	}

	//Advance to, if owned (Property from atlantic)
	if ((e->getAdvanceToIfOwned() != -1) && e->getOwner() == playerOnTurn) {
		playerOnTurn->setPos(e->getAdvanceToIfOwned());
		playerMoved = true;
	}

	if (!wait) {
		movedPlayer();
	}
}

void CapiServerGame::movedPlayer() {
	if (depts.size() > 0) {
		qDebug() << "Bug: going on with non finished depts";
		return;
	}

	//After estate handling
	if (playerMoved) {
		waitForMove();
	} else {
		if (playerOnTurn->getInJail()) {
			endTurn();
		} else {
			wasDouble();
		}
	}
}
void CapiServerGame::endTurn() {
	int eid = -1;
	if (playerOnTurn != 0) {
		//Turn endet. next player gets turn
		playerOnTurn->setHasTurn(false);
		sendToAll(playerOnTurn->getUpdate("hasturn"));
		eid = playerOnTurn->getPos();
	}

	if (getNumPlayingPlayers() > 1) {
		//Begin of a new turn
		do {
			//Get a non bankrupt player
			playerNum = (playerNum+1)%players.count();
			playerOnTurn = (CapiServerPlayer*) players.value(playerNum);
		} while (playerOnTurn->getBankrupt());

		playerOnTurn->setHasTurn(true);
		playerOnTurn->resetDoubles();
		sendToAll(createDisplayMsg(QString("Turn goes to %1").arg(playerOnTurn->getName()),eid));
		if (playerOnTurn->getInJail()) {
			waitForJail();
		} else {
			waitForRoll();
		}
	} else {
		//Game ended
		turnState = -1; //Game over

		setStatus("end");
		sendToAll(getUpdate("status"));
		CapiServerPlayer* p = 0;
		foreach(CapiPlayer* q, players) {
			if (!q->getBankrupt()) p = (CapiServerPlayer*) q;
		}
		if (p == 0) {
			sendToAll(createDisplayMsg("No winner", -1));
		} else
			sendToAll(createDisplayMsg(QString("%1 wins with a furtune of %2").arg(p->getName()).arg(assetsForPlayer(p)), -1, false, false, createButton(".gx", "Exit game")));
	}
}

void CapiServerGame::wasDouble() {
	if (r1 == r2) {
		turnState = 0;
		playerOnTurn->setCanRoll(true);
		sendToAll(createDisplayMsg(QString("%1 can roll again").arg(playerOnTurn->getName()), playerOnTurn->getPos()));
		sendToAll(playerOnTurn->getUpdate("can_roll"));
	} else {
		endTurn();
	}
}

int CapiServerGame::assetsForPlayer(CapiPlayer* p) {
	int assets = p->getMoney();
	foreach (CapiEstate* e, estates) {
		if (e->getOwner() == p) {
			if (!e->getMortaged()) assets += e->getMortageVal();
			assets += e->getHouses()*e->getSellHousePrice();
		}
	}

	return assets;
}

void CapiServerGame::setCanBuyOrSellHouses(CapiServerGroup* g) {
	bool groupMortgaged = false;
	bool sameOwner = g->getAllEstatesSameOwner();
	int maxH = g->getEstate(0)->getHouses();
	int minH = g->getEstate(0)->getHouses();

	//Get the minimum and maximum number of houses on the group
	//noice if one estate is mortgaged
	for (int i = 0; i < g->getNumEstates(); i++) {
		CapiEstate* e = g->getEstate(i);
		if (e->getHouses() < minH) minH = e->getHouses();
		if (e->getHouses() > maxH) maxH = e->getHouses();
		if (e->getMortaged()) groupMortgaged = true;
	}

	//check for every estate in the group, if we can buy/sell houses
	for (int i = 0; i < g->getNumEstates(); i++) {
		CapiServerEstate* es = (CapiServerEstate*) g->getEstate(i);
		if (!sameOwner || groupMortgaged || (g->getHousePrice() == 0)) {
			es->setCanBuyHouse(false);
			es->setCanSellHouse(false);
		} else {
			es->setCanBuyHouse((es->getHouses() < 5) && (es->getHouses() == minH));
			es->setCanSellHouse((es->getHouses() > 0) && (es->getHouses() == maxH));
		}

		es->setCanToggleMortage(g->getNumHouses() == 0);
		sendToAll(es->getUpdate("can_buy_houses can_sell_houses can_toggle_mortgage"));
	}
}

//Gets the number of all houses owned by given player. if no player is given, it counts all houses
int CapiServerGame::getNumHouses(CapiPlayer* p) {
	bool countAll = (p == 0);
	int count = 0;

	foreach(CapiEstate* e, estates) {
		if (countAll || (e->getOwner() == p)) {
			if (e->getHouses() < 5) count += e->getHouses();
		}
	}

	return count;
}

//Gets the number of all hotels owned by given player. if no player is given, it counts all houses
int CapiServerGame::getNumHotels(CapiPlayer* p) {
	bool countAll = (p == 0);
	int count = 0;

	foreach(CapiEstate* e, estates) {
		if (countAll || (e->getOwner()== p)) {
			if (e->getHouses() == 5) count += e->getHouses();
		}
	}

	return count;
}

//If a player has more depts then assets he is banrupt.
void CapiServerGame::bankruptPlayer(CapiDept* d) {
	CapiServerPlayer* p = (CapiServerPlayer*) d->getFromPlayer();
	p->setBankrupt(true);
	p->setInJail(false);
	CapiServerPlayer* tp = 0;
	int tpid = -1;
	if (d->getToPlayer() != 0) {
		tp = (CapiServerPlayer*) d->getToPlayer();
		tp->setMoney(tp->getMoney()+p->getMoney());
		tpid = tp->getId();
		sendToAll(tp->getUpdate("money"));
	}
	p->setMoney(0);

	QString update = p->getUpdate("bankrupt money jailed")+tp->getUpdate("money");
	//All estate goes to the player that would get the money
	foreach (CapiEstate* f, estates) {
		CapiServerEstate* e = (CapiServerEstate*) f;
		if (e->getOwner() == p) {
			e->setOwner(tp);
			update += e->getUpdate("owner");
		}
	}

	//All cards goees to the player that would get the money
	foreach (CapiCard* d, cards) {
		CapiServerCard* c = (CapiServerCard*) d;
		if (c->getOwner() == p) {
			c->setOwner(tp);
			update += c->getUpdate();
		}
	}

	update += createDisplayMsg(QString("%1 is bankrupt.").arg(p->getName()), playerOnTurn->getPos());

	sendToAll(update);
	d->setPayed(true);

	if (p->getId() == playerOnTurn->getId()) {
		endTurn();
	}
}

int CapiServerGame::getNumPlayingPlayers() {
	int pl = 0;
	foreach(CapiPlayer* p, players)
		if (!p->getBankrupt()) pl++;

	return pl;
}
